@@ -160,16 +160,34 @@ print_history (GPtrArray    *dirs,
     while ((reverse && sd_journal_previous (j) > 0) ||
            (!reverse && sd_journal_next (j) > 0))
       {
+        g_autofree char *ref_str = NULL;
+        g_autofree char *remote = NULL;
+
         /* determine whether to skip this entry */
 
+        ref_str = get_field (j, "REF", error);
+        if (*error)
+          return FALSE;
+
+        /* Appstream pulls are probably not interesting, and they are confusing
+         * since by default we include the Application column which shows up blank.
+         */
+        if (ref_str && ref_str[0] && g_str_has_prefix (ref_str, "appstream"))
+          continue;
+
+        remote = get_field (j, "REMOTE", error);
+        if (*error)
+          return FALSE;
+
+        /* Exclude pull to temp repo */
+        if (remote && remote[0] == '/')
+          continue;
+
         if (dirs)
           {
             gboolean include = FALSE;
             g_autofree char *installation = get_field (j, "INSTALLATION", NULL);
 
-            if (installation && installation[0] == '/')
-              include = TRUE; /* pull to a temp repo */
-
             for (i = 0; i < dirs->len && !include; i++)
               {
                 g_autofree char *name = flatpak_dir_get_name (dirs->pdata[i]);
@@ -217,21 +235,35 @@ print_history (GPtrArray    *dirs,
                      strcmp (columns[k].name, "arch") == 0 ||
                      strcmp (columns[k].name, "branch") == 0)
               {
-                g_autofree char *ref = get_field (j, "REF", error);
-                if (*error)
-                  return FALSE;
+                g_autofree char *value = NULL;
+
+                if (ref_str && ref_str[0] &&
+                    !is_flatpak_ref (ref_str) &&
+                    g_strcmp0 (ref_str, OSTREE_REPO_METADATA_REF) != 0)
+                  g_warning ("Unknown ref in history: %s", ref_str);
+
                 if (strcmp (columns[k].name, "ref") == 0)
-                  flatpak_table_printer_add_column (printer, ref);
-                else
+                  value = g_strdup (ref_str);
+                else if (ref_str && ref_str[0] &&
+                         (g_str_has_prefix (ref_str, "app/") ||
+                          g_str_has_prefix (ref_str, "runtime/")))
                   {
-                    g_auto(GStrv) pref = flatpak_decompose_ref (ref, NULL);
-                    if (strcmp (columns[k].name, "application") == 0)
-                      flatpak_table_printer_add_column (printer, pref ? pref[1] : "");
-                    else if (strcmp (columns[k].name, "arch") == 0)
-                      flatpak_table_printer_add_column (printer, pref ? pref[2] : "");
+                    g_autoptr(FlatpakDecomposed) ref = NULL;
+                    ref = flatpak_decomposed_new_from_ref (ref_str, NULL);
+                    if (ref == NULL)
+                      g_warning ("Invalid ref in history: %s", ref_str);
                     else
-                      flatpak_table_printer_add_column (printer, pref ? pref[3] : "");
+                      {
+                        if (strcmp (columns[k].name, "application") == 0)
+                          value = flatpak_decomposed_dup_id (ref);
+                        else if (strcmp (columns[k].name, "arch") == 0)
+                          value = flatpak_decomposed_dup_arch (ref);
+                        else
+                          value = flatpak_decomposed_dup_branch (ref);
+                      }
                   }
+
+                  flatpak_table_printer_add_column (printer, value);
               }
             else if (strcmp (columns[k].name, "installation") == 0)
               {
@@ -242,9 +274,6 @@ print_history (GPtrArray    *dirs,
               }
             else if (strcmp (columns[k].name, "remote") == 0)
               {
-                g_autofree char *remote = get_field (j, "REMOTE", error);
-                if (*error)
-                  return FALSE;
                 flatpak_table_printer_add_column (printer, remote);
               }
             else if (strcmp (columns[k].name, "commit") == 0)
@@ -362,7 +391,7 @@ print_history (GPtrArray    *dirs,
 #endif
 
 static GDateTime *
-parse_time (const char *opt_since)
+parse_time (const char *since_opt)
 {
   g_autoptr(GDateTime) now = NULL;
   g_auto(GStrv) parts = NULL;
@@ -392,12 +421,12 @@ parse_time (const char *opt_since)
       tm.tm_min = 0;
       tm.tm_sec = 0;
 
-      rest = strptime (opt_since, fmts[i], &tm);
+      rest = strptime (since_opt, fmts[i], &tm);
       if (rest && *rest == '\0')
         return g_date_time_new_local (tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday, tm.tm_hour, tm.tm_min, tm.tm_sec);
     }
 
-  parts = g_strsplit (opt_since, " ", -1);
+  parts = g_strsplit (since_opt, " ", -1);
 
   for (i = 0; parts[i]; i++)
     {
