@@ -84,7 +84,7 @@ gboolean
 flatpak_builtin_info (int argc, char **argv, GCancellable *cancellable, GError **error)
 {
   g_autoptr(GOptionContext) context = NULL;
-  g_autofree char *ref = NULL;
+  g_autoptr(FlatpakDecomposed) ref = NULL;
   g_autoptr(FlatpakDir) dir = NULL;
   g_autoptr(GBytes) deploy_data = NULL;
   g_autoptr(FlatpakDeploy) deploy = NULL;
@@ -105,7 +105,6 @@ flatpak_builtin_info (int argc, char **argv, GCancellable *cancellable, GError *
   gboolean search_all = FALSE;
   gboolean first = TRUE;
   FlatpakKinds kinds;
-  g_auto(GStrv) parts = NULL;
   const char *path;
   g_autofree char *formatted_size = NULL;
   gboolean friendly = TRUE;
@@ -149,8 +148,6 @@ flatpak_builtin_info (int argc, char **argv, GCancellable *cancellable, GError *
   if (deploy == NULL)
     return FALSE;
 
-  parts = g_strsplit (ref, "/", 0);
-
   commit = flatpak_deploy_data_get_commit (deploy_data);
   alt_id = flatpak_deploy_data_get_alt_id (deploy_data);
   origin = flatpak_deploy_data_get_origin (deploy_data);
@@ -194,21 +191,21 @@ flatpak_builtin_info (int argc, char **argv, GCancellable *cancellable, GError *
             print_wrapped (MIN (cols, 80), "\n%s\n", name);
         }
 
-      latest = flatpak_dir_read_latest (dir, origin, ref, NULL, NULL, NULL);
+      latest = flatpak_dir_read_latest (dir, origin, flatpak_decomposed_get_ref (ref), NULL, NULL, NULL);
       if (latest == NULL)
         latest = g_strdup (_("ref not present in origin"));
 
       if (ostree_repo_load_commit (flatpak_dir_get_repo (dir), commit, &commit_v, NULL, NULL))
         {
-          VarCommitRef commit = var_commit_from_gvariant (commit_v);
+          VarCommitRef var_commit = var_commit_from_gvariant (commit_v);
 
-          subject = var_commit_get_subject (commit);
+          subject = var_commit_get_subject (var_commit);
           parent = ostree_commit_get_parent (commit_v);
           timestamp = ostree_commit_get_timestamp (commit_v);
 
           formatted_timestamp = format_timestamp (timestamp);
 
-          commit_metadata = var_commit_get_metadata (commit);
+          commit_metadata = var_commit_get_metadata (var_commit);
           xa_metadata = var_metadata_lookup_string (commit_metadata, "xa.metadata", NULL);
           if (xa_metadata == NULL)
             g_printerr (_("Warning: Commit has no flatpak metadata\n"));
@@ -229,7 +226,7 @@ flatpak_builtin_info (int argc, char **argv, GCancellable *cancellable, GError *
         len = MAX (len, g_utf8_strlen (_("Collection:"), -1));
       len = MAX (len, g_utf8_strlen (_("Installation:"), -1));
       len = MAX (len, g_utf8_strlen (_("Installed:"), -1));
-      if (strcmp (parts[0], "app") == 0)
+      if (flatpak_decomposed_is_app (ref))
         {
           len = MAX (len, g_utf8_strlen (_("Runtime:"), -1));
           len = MAX (len, g_utf8_strlen (_("Sdk:"), -1));
@@ -259,10 +256,10 @@ flatpak_builtin_info (int argc, char **argv, GCancellable *cancellable, GError *
 
       width = cols - (len + 1);
 
-      print_aligned (len, _("ID:"), parts[1]);
-      print_aligned (len, _("Ref:"), ref);
-      print_aligned (len, _("Arch:"), parts[2]);
-      print_aligned (len, _("Branch:"), parts[3]);
+      print_aligned_take (len, _("ID:"), flatpak_decomposed_dup_id (ref));
+      print_aligned (len, _("Ref:"), flatpak_decomposed_get_ref (ref));
+      print_aligned_take (len, _("Arch:"), flatpak_decomposed_dup_arch (ref));
+      print_aligned_take (len, _("Branch:"), flatpak_decomposed_dup_branch (ref));
       if (version)
         print_aligned (len, _("Version:"), version);
       if (license)
@@ -272,7 +269,7 @@ flatpak_builtin_info (int argc, char **argv, GCancellable *cancellable, GError *
         print_aligned (len, _("Collection:"), collection_id);
       print_aligned (len, _("Installation:"), flatpak_dir_get_name_cached (dir));
       print_aligned (len, _("Installed:"), formatted_size);
-      if (strcmp (parts[0], "app") == 0)
+      if (flatpak_decomposed_is_app (ref))
         {
           g_autofree char *runtime = NULL;
           runtime = g_key_file_get_string (metakey,
@@ -281,7 +278,7 @@ flatpak_builtin_info (int argc, char **argv, GCancellable *cancellable, GError *
                                            error);
           print_aligned (len, _("Runtime:"), runtime ? runtime : "-");
         }
-      if (strcmp (parts[0], "app") == 0)
+      if (flatpak_decomposed_is_app (ref))
         {
           g_autofree char *sdk = NULL;
           sdk = g_key_file_get_string (metakey,
@@ -338,7 +335,7 @@ flatpak_builtin_info (int argc, char **argv, GCancellable *cancellable, GError *
       if (opt_show_ref)
         {
           maybe_print_space (&first);
-          g_print ("%s", ref);
+          g_print ("%s", flatpak_decomposed_get_ref (ref));
         }
 
       if (opt_show_origin)
@@ -370,16 +367,10 @@ flatpak_builtin_info (int argc, char **argv, GCancellable *cancellable, GError *
           g_autofree char *runtime = NULL;
           maybe_print_space (&first);
 
-          if (strcmp (parts[0], "app") == 0)
-            runtime = g_key_file_get_string (metakey,
-                                             FLATPAK_METADATA_GROUP_APPLICATION,
-                                             FLATPAK_METADATA_KEY_RUNTIME,
-                                             NULL);
-          else
-            runtime = g_key_file_get_string (metakey,
-                                             FLATPAK_METADATA_GROUP_RUNTIME,
-                                             FLATPAK_METADATA_KEY_RUNTIME,
-                                             NULL);
+          runtime = g_key_file_get_string (metakey,
+                                           flatpak_decomposed_get_kind_metadata_group (ref),
+                                           FLATPAK_METADATA_KEY_RUNTIME,
+                                           NULL);
           g_print ("%s", runtime ? runtime : "-");
         }
 
@@ -388,16 +379,10 @@ flatpak_builtin_info (int argc, char **argv, GCancellable *cancellable, GError *
           g_autofree char *sdk = NULL;
           maybe_print_space (&first);
 
-          if (strcmp (parts[0], "app") == 0)
-            sdk = g_key_file_get_string (metakey,
-                                         FLATPAK_METADATA_GROUP_APPLICATION,
-                                         FLATPAK_METADATA_KEY_SDK,
-                                         NULL);
-          else
-            sdk = g_key_file_get_string (metakey,
-                                         FLATPAK_METADATA_GROUP_RUNTIME,
-                                         FLATPAK_METADATA_KEY_SDK,
-                                         NULL);
+          sdk = g_key_file_get_string (metakey,
+                                       flatpak_decomposed_get_kind_metadata_group (ref),
+                                       FLATPAK_METADATA_KEY_SDK,
+                                       NULL);
           g_print ("%s", sdk ? sdk : "-");
         }
 
@@ -415,33 +400,38 @@ flatpak_builtin_info (int argc, char **argv, GCancellable *cancellable, GError *
           if (!g_file_load_contents (file, cancellable, &data, &data_size, NULL, error))
             return FALSE;
 
-          g_print ("%s", data);
+          flatpak_print_escaped_string (data,
+                                        FLATPAK_ESCAPE_ALLOW_NEWLINES
+                                        | FLATPAK_ESCAPE_DO_NOT_QUOTE);
         }
 
       if (opt_show_permissions || opt_file_access)
         {
-          g_autoptr(FlatpakContext) context = NULL;
+          g_autoptr(FlatpakContext) app_context = NULL;
           g_autoptr(GKeyFile) keyfile = NULL;
           g_autofree gchar *contents = NULL;
 
-          context = flatpak_context_load_for_deploy (deploy, error);
-          if (context == NULL)
+          app_context = flatpak_context_load_for_deploy (deploy, error);
+          if (app_context == NULL)
             return FALSE;
 
           if (opt_show_permissions)
             {
               keyfile = g_key_file_new ();
-              flatpak_context_save_metadata (context, TRUE, keyfile);
+              flatpak_context_save_metadata (app_context, TRUE, keyfile);
               contents = g_key_file_to_data (keyfile, NULL, error);
               if (contents == NULL)
                 return FALSE;
 
-              g_print ("%s", contents);
+              flatpak_print_escaped_string (contents,
+                                            FLATPAK_ESCAPE_ALLOW_NEWLINES
+                                            | FLATPAK_ESCAPE_DO_NOT_QUOTE);
             }
 
           if (opt_file_access)
             {
-              g_autoptr(FlatpakExports) exports = flatpak_context_get_exports (context, parts[1]);
+              g_autofree char *id = flatpak_decomposed_dup_id (ref);
+              g_autoptr(FlatpakExports) exports = flatpak_context_get_exports (app_context, id);
               FlatpakFilesystemMode mode;
 
               mode = flatpak_exports_path_get_mode (exports, opt_file_access);
@@ -458,6 +448,8 @@ flatpak_builtin_info (int argc, char **argv, GCancellable *cancellable, GError *
   if (opt_show_extensions)
     {
       GList *extensions, *l;
+      g_autofree char *ref_arch = flatpak_decomposed_dup_arch (ref);
+      g_autofree char *ref_branch = flatpak_decomposed_dup_branch (ref);
 
       len = MAX (len, g_utf8_strlen (_("Extension:"), -1));
       len = MAX (len, g_utf8_strlen (_("ID:"), -1));
@@ -469,14 +461,14 @@ flatpak_builtin_info (int argc, char **argv, GCancellable *cancellable, GError *
       flatpak_get_window_size (&rows, &cols);
       width = cols - (len + 1);
 
-      extensions = flatpak_list_extensions (metakey, parts[2], parts[3]);
+      extensions = flatpak_list_extensions (metakey, ref_arch, ref_branch);
       for (l = extensions; l; l = l->next)
         {
           FlatpakExtension *ext = l->data;
-          g_autofree const char **subpaths = NULL;
+          g_autofree const char **ext_subpaths = NULL;
           g_autoptr(GBytes) ext_deploy_data = NULL;
           g_autofree char *formatted = NULL;
-          g_autofree char *formatted_size = NULL;
+          g_autofree char *ext_formatted_size = NULL;
           g_autofree char *formatted_commit = NULL;
 
           if (ext->is_unmaintained)
@@ -484,8 +476,8 @@ flatpak_builtin_info (int argc, char **argv, GCancellable *cancellable, GError *
               formatted_commit = g_strdup (_("unmaintained"));
               origin = NULL;
               size = 0;
-              formatted_size = g_strdup (_("unknown"));
-              subpaths = NULL;
+              ext_formatted_size = g_strdup (_("unknown"));
+              ext_subpaths = NULL;
             }
           else
             {
@@ -498,23 +490,23 @@ flatpak_builtin_info (int argc, char **argv, GCancellable *cancellable, GError *
               origin = flatpak_deploy_data_get_origin (ext_deploy_data);
               size = flatpak_deploy_data_get_installed_size (ext_deploy_data);
               formatted = g_format_size (size);
-              subpaths = flatpak_deploy_data_get_subpaths (ext_deploy_data);
-              if (subpaths && subpaths[0] && size > 0)
-                formatted_size = g_strconcat ("<", formatted, NULL);
+              ext_subpaths = flatpak_deploy_data_get_subpaths (ext_deploy_data);
+              if (ext_subpaths && ext_subpaths[0] && size > 0)
+                ext_formatted_size = g_strconcat ("<", formatted, NULL);
               else
-                formatted_size = g_steal_pointer (&formatted);
+                ext_formatted_size = g_steal_pointer (&formatted);
             }
 
           g_print ("\n");
-          print_aligned (len, _("Extension:"), ext->ref);
+          print_aligned (len, _("Extension:"), flatpak_decomposed_get_ref (ext->ref));
           print_aligned (len, _("ID:"), ext->id);
           print_aligned (len, _("Origin:"), origin ? origin : "-");
           print_aligned (len, _("Commit:"), formatted_commit);
-          print_aligned (len, _("Installed:"), formatted_size);
+          print_aligned (len, _("Installed:"), ext_formatted_size);
 
-          if (subpaths && subpaths[0])
+          if (ext_subpaths && ext_subpaths[0])
             {
-              g_autofree char *s = g_strjoinv (",", (char **) subpaths);
+              g_autofree char *s = g_strjoinv (",", (char **) ext_subpaths);
               print_aligned (len, _("Subpaths:"), s);
             }
         }
@@ -532,7 +524,7 @@ flatpak_complete_info (FlatpakCompletion *completion)
   g_autoptr(GPtrArray) dirs = NULL;
   g_autoptr(GError) error = NULL;
   FlatpakKinds kinds;
-  int i, j;
+  int i;
 
   context = g_option_context_new ("");
   if (!flatpak_option_context_parse (context, options, &completion->argc, &completion->argv,
@@ -551,16 +543,12 @@ flatpak_complete_info (FlatpakCompletion *completion)
       for (i = 0; i < dirs->len; i++)
         {
           FlatpakDir *dir = g_ptr_array_index (dirs, i);
-          g_auto(GStrv) refs = flatpak_dir_find_installed_refs (dir, NULL, NULL, opt_arch,
-                                                                kinds, FIND_MATCHING_REFS_FLAGS_NONE, &error);
+          g_autoptr(GPtrArray) refs = flatpak_dir_find_installed_refs (dir, NULL, NULL, opt_arch,
+                                                                       kinds, FIND_MATCHING_REFS_FLAGS_NONE, &error);
           if (refs == NULL)
             flatpak_completion_debug ("find local refs error: %s", error->message);
-          for (j = 0; refs != NULL && refs[j] != NULL; j++)
-            {
-              g_auto(GStrv) parts = flatpak_decompose_ref (refs[j], NULL);
-              if (parts)
-                flatpak_complete_word (completion, "%s ", parts[1]);
-            }
+
+          flatpak_complete_ref_id (completion, refs);
         }
       break;
 
@@ -568,16 +556,12 @@ flatpak_complete_info (FlatpakCompletion *completion)
       for (i = 0; i < dirs->len; i++)
         {
           FlatpakDir *dir = g_ptr_array_index (dirs, i);
-          g_auto(GStrv) refs = flatpak_dir_find_installed_refs (dir, completion->argv[1], NULL, opt_arch,
-                                                                kinds, FIND_MATCHING_REFS_FLAGS_NONE, &error);
+          g_autoptr(GPtrArray) refs = flatpak_dir_find_installed_refs (dir, completion->argv[1], NULL, opt_arch,
+                                                                       kinds, FIND_MATCHING_REFS_FLAGS_NONE, &error);
           if (refs == NULL)
             flatpak_completion_debug ("find remote refs error: %s", error->message);
-          for (j = 0; refs != NULL && refs[j] != NULL; j++)
-            {
-              g_auto(GStrv) parts = flatpak_decompose_ref (refs[j], NULL);
-              if (parts)
-                flatpak_complete_word (completion, "%s ", parts[3]);
-            }
+
+          flatpak_complete_ref_branch (completion, refs);
         }
 
       break;
