@@ -50,6 +50,8 @@ static gboolean opt_default_arch;
 static gboolean opt_supported_arches;
 static gboolean opt_gl_drivers;
 static gboolean opt_list_installations;
+static gboolean opt_print_updated_env;
+static gboolean opt_print_system_only;
 static gboolean opt_user;
 static gboolean opt_system;
 static char **opt_installations;
@@ -74,10 +76,13 @@ static FlatpakCommand commands[] = {
   { N_(" Manage installed applications and runtimes") },
   { "install", N_("Install an application or runtime"), flatpak_builtin_install, flatpak_complete_install },
   { "update", N_("Update an installed application or runtime"), flatpak_builtin_update, flatpak_complete_update },
+  /* Alias upgrade to update to help users of yum/dnf */
+  { "upgrade", NULL, flatpak_builtin_update, flatpak_complete_update, TRUE },
   { "uninstall", N_("Uninstall an installed application or runtime"), flatpak_builtin_uninstall, flatpak_complete_uninstall },
   /* Alias remove to uninstall to help users of yum/dnf/apt */
   { "remove", NULL, flatpak_builtin_uninstall, flatpak_complete_uninstall, TRUE },
   { "mask", N_("Mask out updates and automatic installation"), flatpak_builtin_mask, flatpak_complete_mask },
+  { "pin", N_("Pin a runtime to prevent automatic removal"), flatpak_builtin_pin, flatpak_complete_pin },
   { "list", N_("List installed apps and/or runtimes"), flatpak_builtin_list, flatpak_complete_list },
   { "info", N_("Show info for installed app or runtime"), flatpak_builtin_info, flatpak_complete_info },
   { "history", N_("Show history"), flatpak_builtin_history, flatpak_complete_history },
@@ -165,6 +170,8 @@ static GOptionEntry empty_entries[] = {
   { "supported-arches", 0, 0, G_OPTION_ARG_NONE, &opt_supported_arches, N_("Print supported arches and exit"), NULL },
   { "gl-drivers", 0, 0, G_OPTION_ARG_NONE, &opt_gl_drivers, N_("Print active gl drivers and exit"), NULL },
   { "installations", 0, 0, G_OPTION_ARG_NONE, &opt_list_installations, N_("Print paths for system installations and exit"), NULL },
+  { "print-updated-env", 0, 0, G_OPTION_ARG_NONE, &opt_print_updated_env, N_("Print the updated environment needed to run flatpaks"), NULL },
+  { "print-system-only", 0, 0, G_OPTION_ARG_NONE, &opt_print_system_only, N_("Only include the system installation with --print-updated-env"), NULL },
   { NULL }
 };
 
@@ -193,7 +200,7 @@ no_message_handler (const char     *log_domain,
 }
 
 static GOptionContext *
-flatpak_option_context_new_with_commands (FlatpakCommand *commands)
+flatpak_option_context_new_with_commands (FlatpakCommand *f_commands)
 {
   GOptionContext *context;
   GString *summary;
@@ -203,25 +210,25 @@ flatpak_option_context_new_with_commands (FlatpakCommand *commands)
 
   summary = g_string_new (_("Builtin Commands:"));
 
-  while (commands->name != NULL)
+  while (f_commands->name != NULL)
     {
-      if (!commands->deprecated)
+      if (!f_commands->deprecated)
         {
-          if (commands->fn != NULL)
+          if (f_commands->fn != NULL)
             {
-              g_string_append_printf (summary, "\n  %s", commands->name);
+              g_string_append_printf (summary, "\n  %s", f_commands->name);
               /* Note: the 23 is there to align command descriptions with
                * the option descriptions produced by GOptionContext.
                */
-              if (commands->description)
-                g_string_append_printf (summary, "%*s%s", (int) (23 - strlen (commands->name)), "", _(commands->description));
+              if (f_commands->description)
+                g_string_append_printf (summary, "%*s%s", (int) (23 - strlen (f_commands->name)), "", _(f_commands->description));
             }
           else
             {
-              g_string_append_printf (summary, "\n%s", _(commands->name));
+              g_string_append_printf (summary, "\n%s", _(f_commands->name));
             }
         }
-      commands++;
+      f_commands++;
     }
 
   g_option_context_set_summary (context, summary->str);
@@ -547,7 +554,7 @@ find_similar_command (const char *word,
       if (!commands[i].fn)
         continue;
 
-      int d1 = flatpak_levenshtein_distance (word, commands[i].name);
+      int d1 = flatpak_levenshtein_distance (word, -1, commands[i].name, -1);
       if (d1 < d)
         {
           d = d1;
@@ -560,7 +567,7 @@ find_similar_command (const char *word,
     {
       for (i = 0; entries[k][i].long_name; i++)
         {
-          int d1 = flatpak_levenshtein_distance (word, entries[k][i].long_name);
+          int d1 = flatpak_levenshtein_distance (word, -1, entries[k][i].long_name, -1);
           if (d1 < d)
             {
               d = d1;
@@ -730,6 +737,59 @@ flatpak_run (int      argc,
                       exit (EXIT_SUCCESS);
                     }
                 }
+
+              /* systemd environment generator for system and user installations.
+               * Intended to be used only from the scripts in env.d/.
+               * See `man systemd.environment-generator`. */
+              if (opt_print_updated_env)
+                {
+                  g_autoptr(GPtrArray) installations = g_ptr_array_new_with_free_func (g_free);  /* (element-type GFile) */
+                  GPtrArray *system_installation_locations;  /* (element-type GFile) */
+                  const gchar * const *xdg_data_dirs = g_get_system_data_dirs ();
+                  g_autoptr(GPtrArray) new_dirs = g_ptr_array_new_with_free_func (g_free);  /* (element-type filename) */
+                  g_autofree gchar *new_dirs_joined = NULL;
+
+                  /* Work out the set of installations we want in the environment. */
+                  if (!opt_print_system_only)
+                    {
+                      g_autoptr(GFile) home_installation_location = flatpak_get_user_base_dir_location ();
+                      g_ptr_array_add (installations, g_file_get_path (home_installation_location));
+                    }
+
+                  system_installation_locations = flatpak_get_system_base_dir_locations (NULL, &local_error);
+                  if (local_error != NULL)
+                    {
+                      g_printerr ("%s\n", local_error->message);
+                      exit (1);
+                    }
+
+                  for (gsize i = 0; i < system_installation_locations->len; i++)
+                    g_ptr_array_add (installations, g_file_get_path (system_installation_locations->pdata[i]));
+
+                  /* Get the export path for each installation, and filter out
+                   * ones which are already listed in @xdg_data_dirs. */
+                  for (gsize i = 0; i < installations->len; i++)
+                    {
+                      g_autofree gchar *share_path = g_build_filename (installations->pdata[i], "exports", "share", NULL);
+                      g_autofree gchar *share_path_with_slash = g_strconcat (share_path, "/", NULL);
+
+                      if (g_strv_contains (xdg_data_dirs, share_path) || g_strv_contains (xdg_data_dirs, share_path_with_slash))
+                        continue;
+
+                      g_ptr_array_add (new_dirs, g_steal_pointer (&share_path));
+                    }
+
+                  /* Add the rest of the existing @xdg_data_dirs to the new list. */
+                  for (gsize i = 0; xdg_data_dirs[i] != NULL; i++)
+                    g_ptr_array_add (new_dirs, g_strdup (xdg_data_dirs[i]));
+                  g_ptr_array_add (new_dirs, NULL);
+
+                  /* Print in a format suitable for a system environment generator. */
+                  new_dirs_joined = g_strjoinv (":", (gchar **) new_dirs->pdata);
+                  g_print ("XDG_DATA_DIRS=%s\n", new_dirs_joined);
+
+                  exit (EXIT_SUCCESS);
+                }
             }
 
           if (local_error)
@@ -757,12 +817,28 @@ flatpak_run (int      argc,
     check_environment ();
 
   /* Don't talk to dbus in enter, as it must be thread-free to setns, also
-     skip run/build for performance reasons (no need to connect to dbus). */
+     skip run/build/history for performance reasons (no need to connect to dbus). */
   if (g_strcmp0 (command->name, "enter") != 0 &&
       g_strcmp0 (command->name, "run") != 0 &&
-      g_strcmp0 (command->name, "build") != 0)
+      g_strcmp0 (command->name, "build") != 0 &&
+      g_strcmp0 (command->name, "history") != 0)
     polkit_agent = install_polkit_agent ();
 
+  /* g_vfs_get_default can spawn threads */
+  if (g_strcmp0 (command->name, "enter") != 0)
+    {
+      g_autofree const char *old_env = NULL;
+
+      /* avoid gvfs (http://bugzilla.gnome.org/show_bug.cgi?id=526454) */
+      old_env = g_strdup (g_getenv ("GIO_USE_VFS"));
+      g_setenv ("GIO_USE_VFS", "local", TRUE);
+      g_vfs_get_default ();
+      if (old_env)
+        g_setenv ("GIO_USE_VFS", old_env, TRUE);
+      else
+        g_unsetenv ("GIO_USE_VFS");
+    }
+
   if (!command->fn (argc, argv, cancellable, &error))
     goto out;
 
@@ -836,7 +912,6 @@ main (int    argc,
       char **argv)
 {
   g_autoptr(GError) error = NULL;
-  g_autofree const char *old_env = NULL;
   int ret;
   struct sigaction action;
 
@@ -866,15 +941,6 @@ main (int    argc,
   /* Avoid weird recursive type initialization deadlocks from libsoup */
   g_type_ensure (G_TYPE_SOCKET);
 
-  /* avoid gvfs (http://bugzilla.gnome.org/show_bug.cgi?id=526454) */
-  old_env = g_strdup (g_getenv ("GIO_USE_VFS"));
-  g_setenv ("GIO_USE_VFS", "local", TRUE);
-  g_vfs_get_default ();
-  if (old_env)
-    g_setenv ("GIO_USE_VFS", old_env, TRUE);
-  else
-    g_unsetenv ("GIO_USE_VFS");
-
   if (argc >= 4 && strcmp (argv[1], "complete") == 0)
     return complete (argc, argv);
 
