@@ -423,7 +423,7 @@ flatpak_remote_get_title (FlatpakRemote *self)
 /**
  * flatpak_remote_set_title:
  * @self: a #FlatpakRemote
- * @title: The new title
+ * @title: The new title, or %NULL to unset
  *
  * Sets the repository title of this remote.
  *
@@ -709,7 +709,7 @@ flatpak_remote_get_default_branch (FlatpakRemote *self)
 /**
  * flatpak_remote_set_default_branch:
  * @self: a #FlatpakRemote
- * @default_branch: The new default_branch
+ * @default_branch: The new default_branch, or %NULL to unset
  *
  * Sets the default branch configured for this remote.
  *
@@ -1037,40 +1037,6 @@ flatpak_remote_new_with_dir (const char *name,
   return self;
 }
 
-static FlatpakRemoteType
-repo_finder_to_remote_type (OstreeRepoFinder *repo_finder)
-{
-  if (OSTREE_IS_REPO_FINDER_AVAHI (repo_finder))
-    return FLATPAK_REMOTE_TYPE_LAN;
-  else if (OSTREE_IS_REPO_FINDER_MOUNT (repo_finder))
-    return FLATPAK_REMOTE_TYPE_USB;
-  else
-    return FLATPAK_REMOTE_TYPE_STATIC;
-}
-
-FlatpakRemote *
-flatpak_remote_new_from_ostree (OstreeRemote     *remote,
-                                OstreeRepoFinder *repo_finder,
-                                FlatpakDir       *dir)
-{
-  g_autofree gchar *url = NULL;
-  FlatpakRemotePrivate *priv;
-  FlatpakRemote *self = g_object_new (FLATPAK_TYPE_REMOTE,
-                                      "name", ostree_remote_get_name (remote),
-                                      "type", repo_finder_to_remote_type (repo_finder),
-                                      NULL);
-
-  priv = flatpak_remote_get_instance_private (self);
-  if (dir)
-    priv->dir = g_object_ref (dir);
-
-  url = ostree_remote_get_url (remote);
-  if (url != NULL)
-    flatpak_remote_set_url (self, url);
-
-  return self;
-}
-
 /**
  * flatpak_remote_new:
  * @name: a name
@@ -1215,6 +1181,18 @@ flatpak_remote_commit_filter (FlatpakRemote *self,
   return TRUE;
 }
 
+static void
+_key_file_set_or_unset_string (GKeyFile   *config,
+                               const char *group,
+                               const char *key,
+                               const char *value)
+{
+  if (value != NULL)
+    g_key_file_set_string (config, group, key, value);
+  else
+    g_key_file_remove_key (config, group, key, NULL);
+}
+
 gboolean
 flatpak_remote_commit (FlatpakRemote *self,
                        FlatpakDir    *dir,
@@ -1248,32 +1226,10 @@ flatpak_remote_commit (FlatpakRemote *self,
     g_key_file_set_string (config, group, "url", priv->local_url);
 
   if (priv->local_collection_id_set)
-    {
-      if (priv->local_collection_id != NULL)
-        {
-          g_key_file_set_string (config, group, "collection-id", priv->local_collection_id);
-
-          /* When a collection ID is set, flatpak uses signed per-repo and
-           * per-commit metadata instead of summary signatures. */
-          g_key_file_set_boolean (config, group, "gpg-verify-summary", FALSE);
-        }
-      else
-        {
-          g_autoptr(GError) local_error = NULL;
-          gboolean gpg_verify_value;
-
-          g_key_file_remove_key (config, group, "collection-id", NULL);
-
-          /* Without a collection ID gpg-verify-summary should go back to
-           * matching gpg-verify. */
-          gpg_verify_value = g_key_file_get_boolean (config, group, "gpg-verify", &local_error);
-          if (local_error == NULL)
-            g_key_file_set_boolean (config, group, "gpg-verify-summary", gpg_verify_value);
-        }
-    }
+    _key_file_set_or_unset_string (config, group, "collection-id", priv->local_collection_id);
 
   if (priv->local_title_set)
-    g_key_file_set_string (config, group, "xa.title", priv->local_title);
+    _key_file_set_or_unset_string (config, group, "xa.title", priv->local_title);
 
   if (priv->local_filter_set)
     g_key_file_set_string (config, group, "xa.filter", priv->local_filter ? priv->local_filter : "");
@@ -1291,7 +1247,7 @@ flatpak_remote_commit (FlatpakRemote *self,
     g_key_file_set_string (config, group, "xa.icon", priv->local_icon);
 
   if (priv->local_default_branch_set)
-    g_key_file_set_string (config, group, "xa.default-branch", priv->local_default_branch);
+    _key_file_set_or_unset_string (config, group, "xa.default-branch", priv->local_default_branch);
 
   if (priv->local_main_ref_set)
     g_key_file_set_string (config, group, "xa.main-ref", priv->local_main_ref);
