@@ -28,6 +28,8 @@
 #include <gio/gio.h>
 #include <flatpak-installation.h>
 
+G_BEGIN_DECLS
+
 #define FLATPAK_TYPE_TRANSACTION flatpak_transaction_get_type ()
 #define FLATPAK_TYPE_TRANSACTION_PROGRESS flatpak_transaction_progress_get_type ()
 #define FLATPAK_TYPE_TRANSACTION_OPERATION flatpak_transaction_operation_get_type ()
@@ -146,7 +148,13 @@ struct _FlatpakTransactionClass
                                 const char         *realm,
                                 GVariant           *options,
                                 guint               id);
-  gpointer padding[5];
+  void (*install_authenticator)   (FlatpakTransaction *transaction,
+                                   const char         *remote,
+                                   const char         *authenticator_ref);
+
+  gboolean (*ready_pre_auth) (FlatpakTransaction *transaction);
+
+  gpointer padding[3];
 };
 
 FLATPAK_EXTERN
@@ -174,6 +182,10 @@ FlatpakTransactionOperationType flatpak_transaction_operation_get_operation_type
 FLATPAK_EXTERN
 const char *                    flatpak_transaction_operation_get_ref (FlatpakTransactionOperation *self);
 FLATPAK_EXTERN
+GPtrArray *                     flatpak_transaction_operation_get_related_to_ops (FlatpakTransactionOperation *self);
+FLATPAK_EXTERN
+gboolean                        flatpak_transaction_operation_get_is_skipped (FlatpakTransactionOperation *self);
+FLATPAK_EXTERN
 const char *                    flatpak_transaction_operation_get_remote (FlatpakTransactionOperation *self);
 FLATPAK_EXTERN
 GFile *                         flatpak_transaction_operation_get_bundle_path (FlatpakTransactionOperation *self);
@@ -188,6 +200,10 @@ GKeyFile *                      flatpak_transaction_operation_get_metadata (Flat
 FLATPAK_EXTERN
 GKeyFile *                      flatpak_transaction_operation_get_old_metadata (FlatpakTransactionOperation *self);
 FLATPAK_EXTERN
+const char * const *            flatpak_transaction_operation_get_subpaths (FlatpakTransactionOperation *self);
+FLATPAK_EXTERN
+gboolean                        flatpak_transaction_operation_get_requires_authentication (FlatpakTransactionOperation *self);
+FLATPAK_EXTERN
 const char *                    flatpak_transaction_operation_type_to_string (FlatpakTransactionOperationType kind);
 
 FLATPAK_EXTERN
@@ -213,9 +229,15 @@ FLATPAK_EXTERN
 void                flatpak_transaction_set_disable_related (FlatpakTransaction *self,
                                                              gboolean            disable_related);
 FLATPAK_EXTERN
+void                flatpak_transaction_set_disable_auto_pin  (FlatpakTransaction *self,
+                                                               gboolean            disable_pin);
+FLATPAK_EXTERN
 void                flatpak_transaction_set_reinstall (FlatpakTransaction *self,
                                                        gboolean            reinstall);
 FLATPAK_EXTERN
+void                flatpak_transaction_set_no_interaction (FlatpakTransaction *self,
+                                                            gboolean            no_interaction);
+FLATPAK_EXTERN
 void                flatpak_transaction_set_force_uninstall (FlatpakTransaction *self,
                                                              gboolean            force_uninstall);
 FLATPAK_EXTERN
@@ -227,9 +249,17 @@ void                flatpak_transaction_set_parent_window (FlatpakTransaction *s
 FLATPAK_EXTERN
 const char *        flatpak_transaction_get_parent_window (FlatpakTransaction *self);
 FLATPAK_EXTERN
+void                flatpak_transaction_set_include_unused_uninstall_ops (FlatpakTransaction *self,
+                                                                          gboolean            include_unused_uninstall_ops);
+FLATPAK_EXTERN
+gboolean            flatpak_transaction_get_include_unused_uninstall_ops (FlatpakTransaction *self);
+FLATPAK_EXTERN
 void                flatpak_transaction_add_dependency_source (FlatpakTransaction  *self,
                                                                FlatpakInstallation *installation);
 FLATPAK_EXTERN
+void                flatpak_transaction_add_sideload_repo (FlatpakTransaction  *self,
+                                                           const char          *path);
+FLATPAK_EXTERN
 void                flatpak_transaction_add_default_dependency_sources (FlatpakTransaction *self);
 FLATPAK_EXTERN
 gboolean            flatpak_transaction_run (FlatpakTransaction *transaction,
@@ -287,4 +317,6 @@ gboolean            flatpak_transaction_add_uninstall (FlatpakTransaction *self,
 FLATPAK_EXTERN
 gboolean            flatpak_transaction_is_empty (FlatpakTransaction *self);
 
+G_END_DECLS
+
 #endif /* __FLATPAK_TRANSACTION_H__ */
