@@ -24,6 +24,7 @@
 #include <glib/gi18n-lib.h>
 #include <json-glib/json-glib.h>
 #include "flatpak-oci-registry-private.h"
+#include "flatpak-utils-base-private.h"
 #include "flatpak-utils-http-private.h"
 #include "flatpak-auth-private.h"
 #include "flatpak-dbus-generated.h"
@@ -135,7 +136,7 @@ handle_request_ref_tokens_close (FlatpakAuthenticatorRequest *object,
 
   cancel_basic_auth (auth);
 
-  return TRUE;
+  return G_DBUS_METHOD_INVOCATION_HANDLED;
 }
 
 static void
@@ -215,7 +216,7 @@ handle_request_ref_tokens_basic_auth_reply (FlatpakAuthenticatorRequest *object,
     }
   g_mutex_unlock (&auth->mutex);
 
-  return TRUE;
+  return G_DBUS_METHOD_INVOCATION_HANDLED;
 }
 
 static char *
@@ -306,7 +307,7 @@ cancel_request (FlatpakAuthenticatorRequest *request,
   flatpak_authenticator_request_emit_response (request,
                                                FLATPAK_AUTH_RESPONSE_CANCELLED,
                                                g_variant_builder_end (&results));
-  return TRUE;
+  return G_DBUS_METHOD_INVOCATION_HANDLED;
 }
 
 static gboolean
@@ -323,7 +324,7 @@ error_request_raw (FlatpakAuthenticatorRequest *request,
   flatpak_authenticator_request_emit_response (request,
                                                FLATPAK_AUTH_RESPONSE_ERROR,
                                                g_variant_builder_end (&results));
-  return TRUE;
+  return G_DBUS_METHOD_INVOCATION_HANDLED;
 }
 
 static gboolean
@@ -451,6 +452,7 @@ handle_request_ref_tokens (FlatpakAuthenticator *f_authenticator,
 {
   g_autofree char *request_path = NULL;
   g_autoptr(GError) error = NULL;
+  g_autoptr(GError) anon_error = NULL;
   g_autoptr(AutoFlatpakAuthenticatorRequest) request = NULL;
   const char *auth = NULL;
   gboolean have_auth;
@@ -473,7 +475,7 @@ handle_request_ref_tokens (FlatpakAuthenticator *f_authenticator,
       g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR,
                                              G_DBUS_ERROR_INVALID_ARGS,
                                              _("Not a OCI remote"));
-      return TRUE;
+      return G_DBUS_METHOD_INVOCATION_HANDLED;
     }
   g_variant_lookup (arg_options, "no-interaction", "b", &no_interaction);
 
@@ -484,7 +486,7 @@ handle_request_ref_tokens (FlatpakAuthenticator *f_authenticator,
       g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR,
                                              G_DBUS_ERROR_INVALID_ARGS,
                                              _("Invalid token"));
-      return TRUE;
+      return G_DBUS_METHOD_INVOCATION_HANDLED;
     }
 
   request = flatpak_authenticator_request_skeleton_new ();
@@ -494,7 +496,7 @@ handle_request_ref_tokens (FlatpakAuthenticator *f_authenticator,
                                          &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
-      return TRUE;
+      return G_DBUS_METHOD_INVOCATION_HANDLED;
     }
 
   flatpak_authenticator_complete_request_ref_tokens (f_authenticator, invocation, request_path);
@@ -520,15 +522,14 @@ handle_request_ref_tokens (FlatpakAuthenticator *f_authenticator,
 
       g_debug ("Trying anonymous authentication");
 
-      first_token = get_token_for_ref (registry, ref_data, NULL, &error);
+      first_token = get_token_for_ref (registry, ref_data, NULL, &anon_error);
       if (first_token != NULL)
         have_auth = TRUE;
       else
         {
-          if (g_error_matches (error, FLATPAK_ERROR, FLATPAK_ERROR_NOT_AUTHORIZED))
+          if (g_error_matches (anon_error, FLATPAK_ERROR, FLATPAK_ERROR_NOT_AUTHORIZED))
             {
-              g_debug ("Anonymous authentication failed: %s", error->message);
-              g_clear_error (&error);
+              g_debug ("Anonymous authentication failed: %s", anon_error->message);
 
               /* Continue trying with authentication below */
             }
@@ -538,7 +539,7 @@ handle_request_ref_tokens (FlatpakAuthenticator *f_authenticator,
                * that adding some authentication will fix it. It will just cause a bad UX like
                * described in #3753, so just return the error early.
                */
-              return error_request (request, sender, error);
+              return error_request (request, sender, anon_error);
             }
         }
     }
@@ -582,8 +583,8 @@ handle_request_ref_tokens (FlatpakAuthenticator *f_authenticator,
         }
     }
 
-  if (!have_auth)
-    return error_request (request, sender, error);
+  if (!have_auth && n_refs > 0)
+    return error_request (request, sender, error ? error : anon_error);
 
   g_variant_builder_init (&tokens, G_VARIANT_TYPE ("a{sas}"));
 
@@ -616,7 +617,7 @@ handle_request_ref_tokens (FlatpakAuthenticator *f_authenticator,
                                                FLATPAK_AUTH_RESPONSE_OK,
                                                g_variant_builder_end (&results));
 
-  return TRUE;
+  return G_DBUS_METHOD_INVOCATION_HANDLED;
 }
 
 static gboolean
