@@ -6,9 +6,9 @@ set -euo pipefail
 
 skip_revokefs_without_fuse
 
-echo "1..7"
+echo "1..8"
 
-setup_repo
+INCLUDE_SPECIAL_CHARACTER=1 setup_repo
 install_repo
 
 COMMIT=`${FLATPAK} ${U} info --show-commit org.test.Hello`
@@ -17,40 +17,48 @@ ${FLATPAK} info -rcos  org.test.Hello > info
 
 assert_file_has_content info "^app/org\.test\.Hello/$(flatpak --default-arch)/master test-repo ${COMMIT}"
 
-echo "ok info -rcos"
+ok "info -rcos"
+
+${FLATPAK} info --show-metadata  org.test.Hello > info
+
+# CVE-2023-28101
+assert_file_has_content info "name=org\.test\.Hello"
+assert_file_has_content info "^A=x\\\\x09y"
+
+ok "info --show-metadata"
 
 ${FLATPAK} info --show-permissions  org.test.Hello > info
 
-assert_file_empty info
+assert_file_has_content info "^A=x\\\\x09y"
 
-echo "ok info --show-permissions"
+ok "info --show-permissions"
 
 ${FLATPAK} info --show-location  org.test.Hello > info
 
 assert_file_has_content info "app/org\.test\.Hello/$(flatpak --default-arch)/master/${COMMIT}"
 
-echo "ok info --show-location"
+ok "info --show-location"
 
 ${FLATPAK} info --show-runtime  org.test.Hello > info
 
 assert_file_has_content info "^org\.test\.Platform/$(flatpak --default-arch)/master$"
 
-echo "ok info --show-runtime"
+ok "info --show-runtime"
 
 ${FLATPAK} info --show-sdk  org.test.Hello > info
 
 assert_file_has_content info "^org\.test\.Platform/$(flatpak --default-arch)/master$"
 
-echo "ok info --show-sdk"
+ok "info --show-sdk"
 
 ${FLATPAK} info --show-extensions org.test.Hello > info
 
 assert_file_has_content info "Extension: runtime/org\.test\.Hello\.Locale/$(flatpak --default-arch)/master$"
 
-echo "ok info --show-extensions"
+ok "info --show-extensions"
 
 ${FLATPAK} info --file-access=home org.test.Hello > info
 
 assert_file_has_content info "^hidden$"
 
-echo "ok info --file-access"
+ok "info --file-access"
