@@ -49,7 +49,7 @@ ${FLATPAK} remote-add ${U} oci-registry "oci+http://127.0.0.1:${port}"
 
 images=$(${FLATPAK} remote-ls ${U} --columns=app oci-registry | sort | tr '\n' ' ' | sed 's/ $//')
 assert_streq "$images" "org.test.Hello org.test.Platform"
-echo "ok list remote"
+ok "list remote"
 
 # Pull appstream data
 
@@ -69,13 +69,13 @@ gunzip -c $appstream > appstream-uncompressed
 assert_file_has_content appstream-uncompressed '<id>org\.test\.Hello\.desktop</id>'
 assert_has_file $icondir/64x64/org.test.Hello.png
 
-echo "ok appstream"
+ok "appstream"
 
 # Test that 'flatpak search' works
 ${FLATPAK} search org.test.Hello > search-results
 assert_file_has_content search-results "Print a greeting"
 
-echo "ok search"
+ok "search"
 
 # Replace with the app image with detached icons, check that the icons work
 
@@ -88,7 +88,7 @@ assert_has_file $icondir/64x64/org.test.Hello.png
 new_icon_hash=(md5sum $icondir/64x64/org.test.Hello.png)
 assert_streq $old_icon_hash $new_icon_hash
 
-echo "ok detached icons"
+ok "detached icons"
 
 # Try installing from the remote
 
@@ -97,7 +97,7 @@ ${FLATPAK} ${U} install -y oci-registry org.test.Hello
 run org.test.Hello > hello_out
 assert_file_has_content hello_out '^Hello world, from a sandbox$'
 
-echo "ok install"
+ok "install"
 
 make_updated_app oci
 
@@ -116,7 +116,7 @@ assert_not_streq "$OLD_COMMIT" "$NEW_COMMIT"
 run org.test.Hello > hello_out
 assert_file_has_content hello_out '^Hello world, from a sandboxUPDATED$'
 
-echo "ok update"
+ok "update"
 
 # Remove the app from the registry, check that things were removed properly
 
@@ -131,7 +131,7 @@ assert_not_file_has_content $appstream '<id>org\.test\.Hello\.desktop</id>'
 assert_not_has_file $icondir/64x64/org.test.Hello.png
 assert_not_has_file $icondir/64x64
 
-echo "ok appstream change"
+ok "appstream change"
 
 # Change the remote to a non-OCI remote, check that we cleaned up
 
@@ -149,7 +149,7 @@ assert_not_has_file $base/oci/oci-registry.index.gz
 assert_not_has_file $base/oci/oci-registry.summary
 assert_not_has_dir $base/appstream/oci-registry
 
-echo "ok change remote to non-OCI"
+ok "change remote to non-OCI"
 
 # Change it back and refetch
 
@@ -168,9 +168,18 @@ assert_not_has_file $base/oci/oci-registry.index.gz
 assert_not_has_file $base/oci/oci-registry.summary
 assert_not_has_dir $base/appstream/oci-registry
 
-echo "ok delete remote"
+ok "delete remote"
 
-# Try installing the platform via a flatpakref file.
+# Try installing the platform via a flatpakref file. We use a different URL
+# for the runtime repo so we can test that the origin remote is pruned on
+# uninstall below.
+
+cat << EOF > runtime-repo.flatpakrepo
+[Flatpak Repo]
+Version=1
+Url=oci+http://localhost:${port}
+Title=The OCI Title
+EOF
 
 cat << EOF > org.test.Platform.flatpakref
 [Flatpak Ref]
@@ -179,6 +188,7 @@ Name=org.test.Platform
 Branch=master
 Url=oci+http://127.0.0.1:${port}
 IsRuntime=true
+RuntimeRepo=file://$(pwd)/runtime-repo.flatpakrepo
 EOF
 
 ${FLATPAK} ${U} install -y --from ./org.test.Platform.flatpakref
@@ -188,7 +198,7 @@ assert_file_has_content remotes-list '^platform-origin'
 
 assert_has_file $base/oci/platform-origin.index.gz
 
-echo "ok install via flatpakref"
+ok "install via flatpakref"
 
 # Uninstall, check that the origin remote was pruned, and files were
 # cleaned up properly
@@ -196,11 +206,11 @@ echo "ok install via flatpakref"
 ${FLATPAK} ${U} -y uninstall org.test.Platform
 
 ${FLATPAK} remotes > remotes-list
-assert_not_file_has_content remotes '^platform-origin'
+assert_not_file_has_content remotes-list '^platform-origin'
 
 assert_not_has_file $base/oci/platform-origin.index.gz
 
-echo "ok prune origin remote"
+ok "prune origin remote"
 
 # Install from a (non-OCI) bundle, check that the repo-url is respected
 
@@ -213,7 +223,7 @@ assert_file_has_content remotes-list "^platform-origin.*[ 	]oci+http://127\.0\.0
 
 assert_has_file $base/oci/platform-origin.index.gz
 
-echo "ok install via bundle"
+ok "install via bundle"
 
 # Install an app from a bundle
 
@@ -226,7 +236,7 @@ assert_file_has_content remotes-list "^hello-origin.*[ 	]oci+http://127\.0\.0\.1
 
 assert_has_file $base/oci/hello-origin.index.gz
 
-echo "ok app install via bundle"
+ok "app install via bundle"
 
 # Install an updated app bundle with a different origin
 
@@ -240,4 +250,4 @@ assert_file_has_content remotes-list "^hello-origin.*[ 	]http://127\.0\.0\.1:${p
 
 assert_not_has_file $base/oci/hello-origin.index.gz
 
-echo "ok change remote origin via bundle"
+ok "change remote origin via bundle"
