@@ -24,7 +24,7 @@ set -euo pipefail
 skip_without_bwrap
 skip_revokefs_without_fuse
 
-echo "1..37"
+echo "1..41"
 
 #Regular repo
 setup_repo
@@ -77,7 +77,7 @@ fi
 
 # Remove new appstream branch so we can test deploying the old one
 rm -rf repos/test/refs/heads/appstream2
-ostree summary -u --repo=repos/test ${FL_GPGARGS}
+${FLATPAK} build-update-repo ${BUILD_UPDATE_REPO_FLAGS-} --no-update-appstream ${FL_GPGARGS} repos/test
 
 flatpak ${U} --appstream update test-repo
 
@@ -326,7 +326,7 @@ else
 fi
 
 ostree init --repo=repos/test-copy --mode=archive-z2 ${copy_collection_args}
-${FLATPAK} build-commit-from --end-of-life=Reason1 --src-repo=repos/test repos/test-copy app/org.test.Hello/$ARCH/master
+${FLATPAK} build-commit-from --no-update-summary --end-of-life=Reason1 --src-repo=repos/test repos/test-copy app/org.test.Hello/$ARCH/master
 update_repo test-copy ${COPY_COLLECTION_ID}
 
 # Ensure we have no eol app in appdata
@@ -363,7 +363,8 @@ ${FLATPAK} ${U} remote-ls -d -a test-repo > remote-ls-log
 assert_file_has_content remote-ls-log "app/org\.test\.Hello/.*eol=Reason2"
 
 ${FLATPAK} ${U} update -y org.test.Hello > update-log
-assert_file_has_content update-log "org\.test\.Hello.*Reason2"
+assert_file_has_content update-log "org\.test\.Hello.*end-of-life"
+assert_file_has_content update-log "Reason2"
 
 ${FLATPAK} ${U} info org.test.Hello > info-log
 assert_file_has_content info-log "End-of-life: Reason2"
@@ -387,7 +388,7 @@ else
 fi
 
 ostree init --repo=repos/test-rebase --mode=archive-z2 ${rebase_collection_args}
-${FLATPAK} build-commit-from --src-repo=repos/test ${FL_GPGARGS} repos/test-rebase app/org.test.Hello/$ARCH/master runtime/org.test.Hello.Locale/$ARCH/master
+${FLATPAK} build-commit-from --no-update-summary --src-repo=repos/test ${FL_GPGARGS} repos/test-rebase app/org.test.Hello/$ARCH/master runtime/org.test.Hello.Locale/$ARCH/master
 update_repo test-rebase ${REBASE_COLLECTION_ID}
 
 flatpak remote-add ${U} --gpg-import=${FL_GPG_HOMEDIR}/pubring.gpg test-rebase "http://127.0.0.1:${port}/test-rebase"
@@ -399,8 +400,9 @@ ${CMD_PREFIX} flatpak run --command=bash org.test.Hello -c 'echo foo > $XDG_DATA
 assert_has_dir $HOME/.var/app/org.test.Hello
 assert_has_file $HOME/.var/app/org.test.Hello/data/a-file
 
-${FLATPAK} build-commit-from --end-of-life-rebase=org.test.Hello=org.test.NewHello --src-repo=repos/test ${FL_GPGARGS} repos/test-rebase app/org.test.Hello/$ARCH/master runtime/org.test.Hello.Locale/$ARCH/master
+${FLATPAK} build-commit-from --no-update-summary --end-of-life-rebase=org.test.Hello=org.test.NewHello --src-repo=repos/test ${FL_GPGARGS} repos/test-rebase app/org.test.Hello/$ARCH/master runtime/org.test.Hello.Locale/$ARCH/master
 GPGARGS="${FL_GPGARGS}" $(dirname $0)/make-test-app.sh repos/test-rebase org.test.NewHello master "${REBASE_COLLECTION_ID}" "NEW" > /dev/null
+update_repo test-rebase
 
 ${FLATPAK} ${U} update -y org.test.Hello
 
@@ -424,11 +426,78 @@ ${FLATPAK} ${U} uninstall -y org.test.NewHello org.test.Platform
 
 ok "eol-rebase"
 
+# Remove any pin of the runtime from an earlier test
+${FLATPAK} ${U} pin --remove runtime/org.test.Platform/$ARCH/master 2>/dev/null || true
+
+EXPORT_ARGS="--end-of-life=Reason3" make_updated_runtime
+
+${FLATPAK} ${U} install -y test-repo org.test.Hello
+${FLATPAK} ${U} list -d > list-log
+assert_file_has_content list-log "org\.test\.Hello"
+assert_file_has_content list-log "org\.test\.Platform"
+
+${FLATPAK} ${U} uninstall -y org.test.Hello
+
+${FLATPAK} ${U} list -d -a > list-log
+assert_not_file_has_content list-log "org\.test\.Hello"
+assert_not_file_has_content list-log "org\.test\.Platform"
+assert_not_file_has_content list-log "org\.test\.Platform.Locale"
+
+ok "eol runtime uninstalled with app"
+
+${FLATPAK} ${U} install -y test-repo org.test.Hello
+
+${FLATPAK} ${U} info org.test.Platform > info-log
+assert_file_has_content info-log "End-of-life: Reason3"
+
+assert_has_dir $FL_DIR/runtime/org.test.Platform/$ARCH/master/active/files
+
+# Update the app to a different runtime branch
+make_updated_runtime "" "" "mainline" ""
+make_updated_app "" "" "" "UPDATED99" "" "mainline"
+
+${FLATPAK} ${U} update -y org.test.Hello
+
+# The previous runtime should have been removed during the update
+assert_has_dir $FL_DIR/runtime/org.test.Platform/$ARCH/mainline/active/files
+assert_not_has_dir $FL_DIR/runtime/org.test.Platform/$ARCH/master/active/files
+
+# Revert things for future tests
+EXPORT_ARGS="" make_updated_runtime
+make_updated_app "" "" "" "UPDATED100" "" "master"
+${FLATPAK} ${U} uninstall -y --all
+ostree refs --repo=repos/test --delete runtime/org.test.Platform/$ARCH/mainline
+ostree refs --repo=repos/test --delete runtime/org.test.Platform.Locale/$ARCH/mainline
+update_repo
+
+ok "eol runtime uninstalled on app update to different runtime"
+
+${FLATPAK} ${U} install -y test-repo org.test.Hello
+
+# Runtime isn't EOL at time of app uninstall, so it's left alone
+${FLATPAK} ${U} uninstall -y org.test.Hello
+assert_has_dir $FL_DIR/runtime/org.test.Platform/$ARCH/master/active/files
+
+EXPORT_ARGS="--end-of-life=Reason4" make_updated_runtime
+${FLATPAK} ${U} update -y org.test.Platform
+${FLATPAK} ${U} info org.test.Platform > info-log
+assert_file_has_content info-log "End-of-life: Reason4"
+
+# Now that the runtime is EOL and unused it should be uninstalled by the update command
+${FLATPAK} ${U} update -y
+assert_not_has_dir $FL_DIR/runtime/org.test.Platform/$ARCH/master/active/files
+
+# Revert things for future tests
+EXPORT_ARGS="" make_updated_runtime
+${FLATPAK} ${U} uninstall -y --all
+
+ok "eol runtime uninstalled during update run"
+
 ${FLATPAK} ${U} install -y test-repo org.test.Platform
 
 port=$(cat httpd-port)
 UPDATE_REPO_ARGS="--redirect-url=http://127.0.0.1:${port}/test-gpg3 --gpg-import=${FL_GPG_HOMEDIR2}/pubring.gpg" update_repo
-GPGPUBKEY="${FL_GPG_HOMEDIR2}/pubring.gpg" GPGARGS="${FL_GPGARGS2}" setup_repo_no_add test-gpg3 org.test.Collection.test master
+SRC_RUNTIME_REPO="test" GPGPUBKEY="${FL_GPG_HOMEDIR2}/pubring.gpg" GPGARGS="${FL_GPGARGS2}" setup_repo_no_add test-gpg3 org.test.Collection.test master
 
 ${FLATPAK} ${U} update -y org.test.Platform
 # Ensure we have the new uri
@@ -442,12 +511,21 @@ update_repo test-gpg3 org.test.Collection.test
 ${FLATPAK} ${U} install -y test-repo org.test.Hello
 assert_file_has_content $FL_DIR/app/org.test.Hello/$ARCH/master/active/files/bin/hello.sh UPDATED
 
+# Switch back to the old url to unconfuse other tests
+UPDATE_REPO_ARGS="--redirect-url=" update_repo
+${FLATPAK} ${U} remote-modify --url="http://127.0.0.1:${port}/test" test-repo
+
+# Also remove app so we can install the older one from the previous repo
+${FLATPAK} ${U} uninstall -y org.test.Hello
+
 ok "redirect url and gpg key"
 
+${FLATPAK} ${U} install -y -v test-repo org.test.Hello
+
 # Test https://github.com/flatpak/flatpak/issues/3222
 mkdir -p $FL_DIR/repo/refs/mirrors/org.test.Collection.test/app/org.test.Hello/$ARCH/
 cp $FL_DIR/repo/refs/remotes/test-repo/app/org.test.Hello/$ARCH/master $FL_DIR/repo/refs/mirrors/org.test.Collection.test/app/org.test.Hello/$ARCH/
-make_updated_app test-gpg3 org.test.Collection.test master UPDATE2
+make_updated_app test org.test.Collection.test master UPDATE2
 ${FLATPAK} ${U} update -y org.test.Hello
 assert_not_has_file $FL_DIR/repo/refs/mirrors/org.test.Collection.test/app/org.test.Hello/$ARCH/master
 assert_has_file $FL_DIR/repo/refs/remotes/test-repo/app/org.test.Hello/$ARCH/master
@@ -577,6 +655,9 @@ setup_repo
 
 ok "uninstall with missing remote"
 
+# Remove any pin of the runtime from an earlier test
+${FLATPAK} ${U} pin --remove runtime/org.test.Platform/$ARCH/master 2>/dev/null || true
+
 ${FLATPAK} ${U} list -a --columns=application > list-log
 assert_file_has_content list-log "org\.test\.Platform"
 
@@ -587,6 +668,34 @@ assert_not_file_has_content list-log "org\.test\.Platform"
 
 ok "uninstall --unused"
 
+${FLATPAK} ${U} install -y test-repo org.test.Platform
+
+${FLATPAK} ${U} list -a --columns=application > list-log
+assert_file_has_content list-log "org\.test\.Platform"
+
+# Check that the runtime won't be removed if it's pinned
+# (which happens during the install above)
+${FLATPAK} ${U} pin > pins
+assert_file_has_content pins "runtime/org\.test\.Platform/$ARCH/master"
+NUM_PINS=$(cat pins | wc -l)
+if [ $NUM_PINS -ne 1 ]; then
+    assert_not_reached "There should only be one pinned runtime"
+fi
+rm pins
+
+${FLATPAK} ${U} uninstall -y --unused
+
+${FLATPAK} ${U} list -a --columns=application > list-log
+assert_file_has_content list-log "org\.test\.Platform"
+
+# Remove the pin and try again
+${FLATPAK} ${U} pin --remove "runtime/org.test.Platform/$ARCH/master"
+${FLATPAK} ${U} uninstall -y --unused
+${FLATPAK} ${U} list -a --columns=application > list-log
+assert_not_file_has_content list-log "org\.test\.Platform"
+
+ok "uninstall --unused ignores pinned runtimes"
+
 # Test that remote-ls works in all of the following cases:
 # * system remote, and --system is used
 # * system remote, and --system is omitted
@@ -622,7 +731,7 @@ fi
 ok "remote-ls"
 
 # Test that remote-ls can take a file:// URI
-ostree --repo=repos/test summary -u
+${FLATPAK} build-update-repo  ${BUILD_UPDATE_REPO_FLAGS-} --no-update-appstream repos/test
 ${FLATPAK} remote-ls file://`pwd`/repos/test > repo-list
 assert_file_has_content repo-list "org\.test\.Hello"
 
